--
-- href.hs
--
-- Copyright (c) 2003 Minero Aoki <aamine@loveruby.net>
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.
-- For details of the GNU LGPL, see the file "COPYING".
--

module Main where

import System
import System.IO
import System.Directory
import Control.Monad
import Text.Regex
import Data.Char
import Data.List

import Href.TextUtils
import Href.FileUtils
import Href.HrefUtils
import Href.Config

main :: IO ()
main = do
    args <- getArgs
    case args of
      []         -> lookupModule ""
      [name]     -> if isUpper (headChar name) then lookupModule name
                                               else lookupFunction name
      [mod,func] -> lookupFunction2 mod func
      _          -> do hPutStrLn stderr "0..2 arguments must be given"
                       exitWith $ ExitFailure 1

lookupModule :: String -> IO ()
lookupModule name = do
    pathes <- glob0 hrefPrefix
        (mkRegex $ "^" ++ hrefPrefix ++ "/" ++ encodePath name ++ "[^/]*$")
    case pathes of
      []     -> notFound "module" name
      [path] -> do es <- dirEntries path
                   putStrLn $ "---- module " ++ (moduleName path)
                   putList $ map (decodePath) es
      _      -> putList $ map (moduleName) pathes

dirEntries :: String -> IO [String]
dirEntries path = do
    es <- getDirectoryContents path
    return $ filter (\e -> case e of
                             "." -> False
                             ".." -> False
                             _ -> True) es

lookupFunction :: String -> IO ()
lookupFunction name = do
    pathes <- glob0 hrefPrefix
                    (mkRegex $ "^" ++ hrefPrefix ++ "/.*/" ++ encodePath name)
    lookupFunction' pathes "" name

lookupFunction2 :: String -> String -> IO ()
lookupFunction2 m f = do
    pathes <- glob0 hrefPrefix
      (mkRegex $ "^"++hrefPrefix++"/"++encodePath m++ "[^/]*/"++encodePath f)
    lookupFunction' pathes m f

lookupFunction' :: [String] -> String -> String -> IO ()
lookupFunction' pathes m f = do
    case pathes of
      []     -> notFound "function" $ m++" "++f
      [path] -> describeFunction path
      _      -> case filter (\p -> functionName p == f) pathes of
                  [path] -> describeFunction path
                  _      -> putList $ map (fullName) pathes

describeFunction :: String -> IO ()
describeFunction path = do
    putStrLn $ fullName path
    cs <- readFile path
    putStr cs

putList :: [String] -> IO ()
putList xs = do putList' hrefMaxColumns $ sort xs
                putStrLn ""

putList' :: Int -> [String] -> IO ()
putList' col [] = return ()
putList' col (x:xs)
    | col > 0   = do putStr (x ++ " ")
                     putList' (col - length x - 1) xs
    | otherwise = do putStrLn ""
                     putStr (x ++ " ")
                     putList' hrefMaxColumns xs

notFound :: String -> String -> IO ()
notFound kind name = do
    hPutStrLn stderr ("no such " ++ kind ++ ": " ++ name)
    exitWith $ ExitFailure 1
